//Main header file for the Legend MUD engine. Written by Barghest (Todd Allen),
//toddmallen@gmail.com.

//Updates handled by the main engine, mainly consisting of sending signals to
//modules, processing data from players, and sending output to them.

/*******************************************************************************
 * Legend is free software licensed under the GNU General Public License (GPL),*
 * available in the attached COPYING and/or license.txt file in the docs 	   *
 * directory, or at www.fsf.org.																   *
 * This program is provided with NO WARRANTY of any kind and is used or run    *
 * entirely at the user's own risk.											   *
 ******************************************************************************/

#ifndef __cplusplus
#error Legend must be compiled with a C++ compiler. See README in the /docs \
directory for further information.
#endif

#include <string>
#include <vector>
#include <cctype>
#include <fstream>
#include <sstream>
#include <time.h>
#include <sys/time.h>
using namespace std;

//System-wide settings for conditional compilation.
#define DEBUG
#define USECRYPT

const int MAX_STRING_INPUT = 40000; //Maximum length of an input string.
const int PULSE_PER_SECOND = 4; //Pulses per second.

void log_console(string logstring);

//Updating the MUD and its data.
namespace UPDATE
{
	
class Log_Handler;
	
class Update_Handler
{
	private:
	bool shut_down;
	Log_Handler *log_handler;
	void register_sighandlers(void); //Begin signal handling.
	
	public:
	Update_Handler(); //{log_console("Initializing update handler"); shut_down = false;}
	~Update_Handler(); //{log_console("Terminating update handler.");}
	void update_mud(void) {} //TODO: Define, placeholder for now
	void shutdown_mud(void) {shut_down = true;} //May want to add some checking here.
	bool check_shutdown(void) {return shut_down;}
	Log_Handler *get_loghandler(void) {return log_handler;}
};

//Main handler for all other objects.
class Mud_Handler
{
	private:
	struct timeval started_time;
	struct timeval current_time;
	struct timeval tick_timer;
	
	public:
	Mud_Handler();
	~Mud_Handler() {}
	string get_time(void) const;
	string get_boot_time(void) const;
	int get_time_secs(void) const;
	int get_boot_time_secs(void) const;
	void set_tick_timer(void);
#ifdef USECRYPT
	bool check_crypt(string pass, string name, string hash); //Check if the hash of pass and name match hash.
#else
	bool check_pass(string pass, string test); //Check an unencrypted password.
#endif
};
//All functions defined in update.cpp.
class Log_Handler
{
	private:
	string log_name;
	bool logging_status;
	
	public:
	Log_Handler(); //{logging_status = false; set_logfile();} //Open and initiate log file.
	~Log_Handler();
	void log_standard(string tolog, int recv_level); //Log to MUD (stdout for now) and log file.
	//Log to file only. Better for log messages which may be important to record
	//but would be annoying or spammy if logged to the MUD.
	void log_silent(string tolog);
	void log_error(string tolog, int recv_level, int severity); //Log an error.
	void log_error_silent(string tolog, int severity); //Like log_silent for errors.
	void set_logfile(void);
	bool get_logging_status(void) const {return logging_status;}
};
}

//Error codes for global_errcode.
const int ERROR_NONE = 0;
const int ERROR_MINOR = 1;
const int ERROR_MAJOR = 2;
const int ERROR_CRITICAL = 3;
const int ERROR_FATAL = 4;

const string LOGPATH = "../log";
const int MAX_LOG_DUMP = 100;

//Error handling.
namespace ERROR
{

//Global error code information. Generally to use in passing error information
//from modules to the engine when not serious enough to warrant throwing an
//exception.
class Global_Errcode
{
	private:
	int err_type;
	string message;
	
	public:
	void set_error(const int type); //TODO: Define
	int get_errtype(void) const {return err_type;}
	void set_message(const string newmsg) {message = newmsg;}
	string get_message(void) const {return message;}
	void reset_error(void) {message.clear(); err_type = ERROR_NONE;}
};

class E_Handler
{
	public:
	string message;
	E_Handler() {log_console("ERROR: E_Handler initialized without message.");}
	~E_Handler() {}
	E_Handler(string err_msg) {message = err_msg;}
	void clear_exception(void) {message.clear();}
};

//Buffer information to be sent to a player.
//class Send_Buffer
//{
//	private:
//	string to_send;
//	

} //end namespace ERROR

//Various miscellaneous but (hopefully) useful utility functions and templates.
namespace UTILS
{
	string strip_whitespace(string in_str);
	bool file_exists(const string &filename);
	template<class type_out, class type_in>
    type_out convert(const type_in in)
    {
        //using std::stringstream;
        stringstream ss;
        ss << in;
        type_out buf;
        ss >> buf;
        return buf;
    }
//    template<class to_check, class check_val>
//    bool assert_eq(const to_check assertion, const check_val val)
//    {
//    	if(assertion == val)
//    	{
//    		return true;
//    	}
//    	log_string("Assertion failed!");
//    	return false;
//    }
//    template<class to_check, class check_val>
//    bool assert_gt(const to_check assertion, const check_val val)
//    {
//    	if(assertion > val)
//    	{
//    		return true;
//    	}
//    	
//    	return false;
//    }
} //end namespace UTILS
